/*
 *   Simplex method
 *
 *       Authors:  S.Yu.Orevkov, Yu.P.Orevkov
 *       Version:  1.1
 *   Last update:  October 9, 2010
 *
 */

#include <stdio.h>
#include <stdlib.h>
#define YES 1
#define NO  0

#define _DOUBLE

#ifdef _DOUBLE 
#define real       double
#define real_fmt   "%lf"
#define _Zero      1E-11
#define Infty      1E15
#else
#define real       float
#define real_fmt   "%f"
#define _Zero      1E-6
#define Infty      1E15
#endif

real **A;               /* the matrix */

real Zero=_Zero;        /* absolute precision */

int  m,n,n_init;        /* n_init may differ from n  */
                        /* only in Gomory's method   */
                        /* which is not implemented  */
                        /* yet in this version       */
int n_var, n_eq;

char **var;
int  *base;
int  *iimin=NULL;

char   out_fmt[15]="%15.12lf";  /* output format for the solution */
char matrix_fmt[15]=" %6.2lf";  /* output format for the matrix   */

                          /* Command line options */
int optT=NO, optM=NO, optI=NO, optG=NO, optN=NO, optD=NO;

int Dual=NO;
int trace_m;

#define IsZero(x)     ( (x)<Zero && (x)>-Zero )
#define IsNotZero(x)  ( (x)>Zero || (x)<-Zero )

void matrix_transform(int i0, int i1, int m);
int find_imin( int j );

void outS(void);
void outA(void);

/*===========================================================*/

void simplex( void ){
  int i,j,mm,imin;

  /* minimize Sum( -A[0][j]*xj, j=1..n );             */
  /*  Sum( A[i][j]*xj, j=1..n ) = A[i][0],  i=1,...,m */
  /*  xj >= 0,  j=1,...,n                             */
  /*  A[i][0] >= 0                                    */


  /* Phase one: search of a feasible solution */

  trace_m = mm = m+1;

  for( j=0; j<=n; j++ ){ /*initialize the auxiliary obj.function */
    A[mm][j]=0;
    for( i=1; i<=m; i++ ) A[mm][j] += A[i][j];
  }

  while( YES ){
    if(optT){ puts("\nIteration of the phase one"); outA(); }

    for( j=1; j<=n; j++ ){ if( A[mm][j] > Zero )break; }

    if( j>n ){
      if( IsNotZero( A[mm][0] ) ){
        puts(Dual?"unbounded": "no feasible solutions\n"); exit(1);
      }
      else break;       /* a feasible solution is found */
    }
    imin = find_imin(j);
    if( imin==0 ){ puts("(1) very strange..."); exit(1); }
    matrix_transform(imin,j,mm);
  }

  /* Phase two: search of an optimal solution */

  trace_m = m;

  puts( Dual ?
    "A feasible solution of the dual problem is found.\n" :
    "A feasible solution is found."
  );

  while( YES ){
    if(optT){ puts("\nIteration of the phase two"); outA(); }

    for( j=1; j<=n; j++ ){ if( A[0][j] > Zero )break; }

    if( j>n ) return;        /* success! */

    if( (imin=find_imin(j)) == 0 ){
      puts(Dual?"no feasible solution": "unbounded"  ); exit(1);
    }
    matrix_transform(imin,j,m);
  }
}
/*===========================================================*/

int find_imin( int j ){
    int i,k,namin=0,imin=0;
    real a,amin=Infty;

    for( i=1; i<=m; i++ ){
      if( A[i][j] <= Zero )continue;
      if( (a=A[i][0]/A[i][j]) < amin-Zero ){ namin=1; amin=a; imin=i; }
      else if( a < amin+Zero ) namin++; 
    }
    if( namin <= 1 )return imin;

                                             /* Degenerate problem  */
                                             /* (min is not unique).*/
                                             /* lexicographic order */
    { int jj, kk;
      if( iimin==NULL ) iimin = (int *)malloc(m*sizeof(int));
      k=0;
      for( i=1; i<=m; i++ ){
        if( A[i][j] <= Zero )continue;
        if( A[i][0]/A[i][j] < amin+Zero )iimin[k++]=i;
      }

      if( optT ){
        printf(" j=%d namin=%d iimin[] = ",j,namin);
        for( i=0; i<k; i++ )printf(" %d ",iimin[i] );putchar('\n');
      }
      if( k!= namin ){ puts("(2) very strange..."); exit(1); }

      for( jj=1; jj<=n; jj++ ){ if(jj==j)continue;
        amin=Infty;
        for( k=0; k<namin; k++ ){ i=iimin[k];
          if( (a=A[i][jj]/A[i][j]) < amin )amin=a;
        }
        kk=0;
        for( k=0; k<namin; k++ ){ i=iimin[k];
          if( A[i][jj]/A[i][j] < amin+Zero )iimin[kk++]=i;

        }

        if( optT ){
          printf("  jj=%d amin= %9.5lf namin=%d iimin[] = ",jj,amin,kk);
          for( i=0; i<kk; i++ )printf(" %d ",iimin[i] );putchar('\n');
        }

        if( (namin=kk)==1 )break;
      }
      return iimin[0];
    }
}

/*===========================================================*/

void matrix_transform( int i0, int j0, int mm ){
  int i,j;
  real a;

  if(optT)printf(" Matrix transform with i0=%d j0=%d\n",i0,j0);

  a=A[i0][j0];
  if( j0 <= n_init )base[j0]=i0;
  for( j=0; j<=n; j++ ){
    if( j!=j0 && base[j] && IsNotZero( A[i0][j] ) )base[j]=0;
    A[i0][j] /= a;
  }
  for( i=0; i<=mm; i++ ){
    if( i==i0 )continue;
    a=A[i][j0];
    for( j=0; j<=n; j++ )A[i][j] -= A[i0][j]*a;
  }
}

/*================================================================*/

void usage( char *s ){
  printf(
  "\nSimplex method\nUsage:\n %s [options] [<input] [>output]\n",s);
  printf("Options:\n -i   Inequalities\n");
  printf(          " -d   Dual problem\n");
  printf(          " -n   No dual problem\n");
  printf(          " -m   print resulting Matrix\n");
  printf(
   " -mnn print resulting Matrix with nn digits after the point\n");
  printf(          " -t   Trace\n");
  printf(          " -pnn absolute Precision is 10^(-nn)\n");
  exit(1);
}

/*================================================================*/

void no_mem(void){ puts("memory allocation fail"); exit(1); }

/*================================================================*/

int main( int argc, char *argv[] ){
  int i,j,j0;
  char *arg;

  for( i=1; i<argc; i++ ){
    arg=argv[i];
    if( arg[0] != '-' )usage(argv[0]);
    switch( arg[1] ){int rc,prec,fail_prec; 
     case 't': case 'T': optT=YES; break;     /* fuller Trace    */
     case 'i': case 'I': optI=YES; break;     /* Inequalities    */
     case 'd': case 'D': optD=YES; break;     /* Dual problem    */
     case 'n': case 'N': optN=YES; break;     /* No dual problem */
     case 'm': case 'M': optM=YES;            /* print Matrix    */
       if( arg[2] ){ rc=sscanf(arg+2,"%d",&prec);
         if( rc==0 )fail_prec=YES;
         else fail_prec = (prec<0 || prec>99);
         if( !fail_prec )sprintf(matrix_fmt+2,"%d.%dlf",prec+4,prec); 
       }
       break;
     case 'p': rc=sscanf(arg+2,"%d",&prec);   /* Precision       */
       if( rc==0 )fail_prec=YES;
       else fail_prec = (prec<0 || prec>99);
       if( fail_prec ){ 
         puts("\nThe option -n requires a number 0...99 after it");
         puts("(without space between -n and the number!)");
       }
       else{ 
         Zero=1.; for(j=0; j<prec; j++)Zero=Zero/10;
         sprintf(out_fmt+1,"%d.%dlf",prec+3,prec);         
       }
       break;
     default: usage(argv[0]);
    }
  }

  if( optD && optN ){
    puts("The options -d and -n cannot be used both"); exit(1);
  }

  if( optD && !optI ){
    puts("The option -d cannot be used without -i"); exit(1);
  }

  scanf("%d", &n_var); scanf("%d", &n_eq); 
  if(n_var < 1){ puts("n_var must be positive"); exit(1); }
  if(n_eq  < 1){ puts("n_eq  must be positive"); exit(1); }

  if( optD )Dual=YES;
  else if( !optN ) Dual = (optI && (n_eq > n_var));

  if( optI ){      /* input matrix is interpreted as inequalities */ 
                   /*   sum( A[i][j]*xj, j=1..n_var ) <= A[i][0]  */

    if( Dual ){ n = n_var + n_eq;   m = n_var; }
    else      { n = n_var + n_eq;   m = n_eq;  }
  }
  else{            /* input matrix is interpreted as equations   */ 
                   /*   sum( A[i][j]*xj, j=1..n_var ) = A[i][0]  */

    if( Dual ){ n = n_eq;   m = n_var; }
    else      { n = n_var;  m = n_eq;  }
  }
  n_init = n;  trace_m = m;

  if( optT && Dual )puts("\nThe dual problem is being solved");
  if(optT)printf("\n m = %d, n = %d\n",m,n);

  A = (real **)malloc( (m+2)*sizeof(real *) ); if(A==NULL)no_mem();
  for(i=0; i<=m+1; i++){
    A[i] = (real *)malloc((n+1)*sizeof(real)); if(A[i]==NULL)no_mem();
  }

  var = (char **)malloc( (n_var+1)*sizeof(char *) ); if(var==NULL)no_mem();
  base = (int *)malloc( (n+1) * sizeof(int) ); if(base==NULL)no_mem();

  for( i=0; i<=n_eq; i++ ){
    for( j=0; j<=n_var; j++ ){
      scanf( real_fmt, ( Dual ? A[j]+i : A[i]+j ) );
      if( Dual )A[j][i]=-A[j][i];
    }
  }

                 /* input the names of variables */

  {
    int c,k; char line[100]; 
    puts("\nVariable names:\n");
    for( j=1; j<=n_var; j++ ){
      var[j]=NULL;
      while( (c=getchar()) != EOF ){
        if( c!=' ' && c!='\t' && c!='\n' )break;
      }
      if(c==EOF)break;
      line[0]=c;
      for( i=1; i<99; i++ ){
        line[i]=c=getchar(); 
        if( c==EOF || c==' ' || c=='\t' || c=='\n' )break;
      }
      line[k=i]=0;
      var[j] = (char *)malloc( (i+1)*sizeof(char) );
      for( i=0; i<=k; i++ )var[j][i] = line[i];
      printf("%d-th variable is %s\n",j,var[j]);
      if(c==EOF)break;
    }
    if( var[j]==NULL )j--;

    if( (k=j)<n_var ){
      for( j=k+1; j<=n_var; j++ ){
        var[j] = (char *)malloc(6*sizeof(char));
        for( i=1; i<6; i++ )var[j][i]=0;
        var[j][0]='X'; sprintf(var[j]+1,"%d",j);
        printf("%d-th variable is %s\n",j,var[j]);
      }
    }
  }

                 /* introduce slack variables i.e.         */
                 /* attach an identity matrix to the right */
  if( optI ){
    for( i=0; i<=m; i++ )for( j=n-m+1; j<=n; j++ )A[i][j] = 0; 
    for( i=1; i<=m; i++ )A[i][n-m+i] = 1;
  }

        /* For each row:                                   */
        /*   make non-negative the right-hand side         */
        /*   and if the rhs is zero then make non-negative */
        /*   the first non-zero element of the row         */

  for( i=1; i<=m; i++ ){
    if( A[i][0] < -Zero ){
      for( j=0; j<=n; j++ )A[i][j] = -A[i][j];
    }
    if( IsZero(A[i][0]) ){
      for( j=1; j<=n; j++ ){
        if( IsNotZero(A[i][j]) )break;
      }
      j0=j;
      if( j0<=n && A[i][j0]<-Zero ){
        for( j=j0; j<=n; j++ )A[i][j] = -A[i][j];
      }
    }
  }

  if(optT){ puts("\n The initial matrix:\n"); outA(); }


  simplex();

  if( optG )puts("\n\nReal solution:\n");
  else puts("\n\nSolution:\n");

  if( optT || optM )outA();
  outS();

  if( !optG )exit(0);
}

/*============================================================*/

void outS( void ){                     /* Output the solution */
  int j; double value;

  if( Dual && (!optI) ){ 
    puts(
"For dual system of equations we dont print the variable values");
    return;
  }
  for( j=1; j<=n_var; j++ ){
    if( Dual )value = -A[0][n-m+j];
    else value = (base[j] ? A[base[j]][0] : 0.);
    printf("%s = ",var[j]); printf( out_fmt, value ); putchar('\n');
  }
  printf("\nThe value of the objective function is  ");
  printf(out_fmt, (double)(Dual ? -A[0][0] : A[0][0]) );
  putchar('\n'); putchar('\n');
} 

/*============================================================*/

void outA( void ){                       /* Output the matrix */
  int i,j;
  for( i=0; i<=trace_m; i++ ){
    for( j=0; j<=n; j++ )printf(matrix_fmt,(double)A[i][j]);

/*
    printf("  ?"); printf(matrix_fmt,A[i][0]); printf("  =");
    printf(matrix_fmt, A[i][1] + A[i][2]*2/7 + A[i][3] + A[i][4]*5/7 + A[i][5]*3/7 + A[i][9]*5/7 + A[i][10]*2/7);
*/

    putchar('\n');
  }
  puts("base:");for(j=1;j<=n;j++)printf("%d ",base[j]);
  putchar('\n');putchar('\n');
}
